import re

# Taken from flask_uuid: http://git.io/vmecV
UUID_RE = re.compile(
    r'^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$')

# Musicbrainz Recording ID
# https://picard.musicbrainz.org/docs/mappings/
ufid = u'UFID:http://musicbrainz.org'


def is_tagged(tags):
    """ Determine whether an MP3 file has been tagged by MusicBrainz Picard. """
    if ufid not in tags:
        return False
    else:
        return re.match(UUID_RE, tags[ufid].data.decode('ascii')) is not None




from mutagen.mp3 import MP3

from . import commands
from .is_tagged import is_tagged


def _process(files, func, args, skip_tagged=True, needs_file=False):
    """ Apply func to ID3 tags of files. """

    for file in files:

        try:
            tags = MP3(file)
        except:
            print("Couldn't read: %s" % file)
            continue

        if (skip_tagged and is_tagged(tags)):
            print("Already tagged: %s " % file)
            continue

        print("Processing: %s" % file)

        if needs_file:
            func(file=file, tags=tags, **args)
        else:
            func(tags=tags, **args)
            tags.save()


def replace(files, **kwargs):
    _process(files, commands.replace, kwargs)


def remove(files, **kwargs):
    _process(files, commands.remove, kwargs)


def rename(files, skip_tagged=True, **kwargs):
    _process(files, commands.rename, kwargs,
             skip_tagged=skip_tagged, needs_file=True)



execfile("tagtool/version.py")

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    import pypandoc
    long_description = pypandoc.convert('Readme.md', 'rst')
except(IOError, ImportError):
    long_description = open('Readme.md').read()

setup(
    name='tagtool',
    packages=['tagtool'],
    version=__VERSION__,
    description='Tagtool allows you to perform mass ID3 tag cleaning operations.',
    long_description=long_description,
    url='https://github.com/dufferzafar/tagtool',
    license='MIT',
    author='Shadab Zafar',
    author_email='dufferzafar0@gmail.com',
    install_requires=[
        'docopt',
        'mutagen'
    ],
    scripts=[
        'bin/tagtool',
        'bin/tagfix'
    ],
    classifiers=[
          'Environment :: Console',
          'License :: OSI Approved :: MIT License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Unix',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
    ],
)




import os
import re

from mutagen.id3 import TextFrame


def replace(tags, pattern, repl='', prune=True, frames=[], frames_skip=[]):
    r"""
    Perform regex replace on tags like artist name, track title etc.

    Usage:
        tagtool replace --pattern=<pattern> [--repl=<replacement>] <file>...
                        [--prune] [--frames=<tags>] [--frames_skip=<tags>]

    Options:
        <file>                       MP3 file(s) to process
        --pattern=<pattern>          Pattern to search for
        --repl=<replacement>         Replacement text
        --prune                      Remove empty frames
        --frames=<frames>            Comma separated list of frames to
                                     restrict replacement to
        --frames_skip=<frames>       Comma separated list of frames to skip

    Examples:
        tagtool replace --pattern='\s+\d+\s+' --repl='' --prune SomeSong.mp3 --frames_skip=TOPE,TIT2
    """

    for key in list(tags.keys()):

        if ((frames and key not in frames)
                or (frames_skip and key in frames_skip)
                or not isinstance(tags[key], TextFrame)):
            continue

        if re.search(pattern, str(tags[key])):
            sub = re.sub(pattern, repl, str(tags[key]))
            tags[key].text = [sub]
            if not sub and prune:
                del tags[key]


def remove(tags, frames, prefix=True):
    r"""
    Remove tag frames.

    Usage:
        tagtool remove  --frames=<tags> <file>...
                        [--prefix]

    Options:
        <file>                       MP3 file(s) to process
        --frames=<frames>            Comma separated list of frames to remove
        --prefix                     Consider the list of frames as prefixes

    Examples:
        tagtool remove --frames=APIC,COMM,USLT --prefix *.mp3
    """

    for key in list(tags.keys()):

        if prefix:
            if key.startswith(tuple(frames)):
                del tags[key]
        else:
            if key in frames:
                del tags[key]


# TODO: Support patterns like
# %title% - %artist%
def rename(file, tags):
    r"""
    Rename a file with it's track title.

    Usage:
        tagtool rename <file>...

    Options:
        <file>                       MP3 file(s) to process

    Examples:
        tagtool rename *.mp3
    """

    new_path = os.path.join(os.path.dirname(file),
                            "%s.mp3" % tags['TIT2'])
    os.rename(file, new_path)
    return new_path
